@@ -2,7 +2,7 @@
    anubisdb.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003, 2004, 2007, 2009 The Anubis Team.
+   Copyright (C) 2003-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -84,7 +84,8 @@ anubis_db_register (const char *dbid,
 
 
 int
-anubis_db_open (char *arg, enum anubis_db_mode mode, void **dptr, char **err)
+anubis_db_open (char *arg, enum anubis_db_mode mode, void **dptr,
+		char const **err)
 {
   struct anubis_db_instance *inst;
   ANUBIS_URL *url;
