@@ -1,7 +1,7 @@
 /*
    anubisusr.h
    
-   Copyright (C) 2004, 2005, 2007, 2008 The Anubis Team.
+   Copyright (C) 2004-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -29,10 +29,10 @@
 #define obstack_chunk_free free
 #include <obstack.h>
 
-#if defined(USE_GNUTLS) && defined(HAVE_GNUTLS_GNUTLS_H)
+#if defined(USE_GNUTLS)
 # include <gnutls/gnutls.h>
 # define HAVE_TLS
-#endif /* USE_GNUTLS and HAVE_GNUTLS_GNUTLS_H */
+#endif /* USE_GNUTLS */
 
 extern char *tls_cafile;
 extern int enable_tls;
