@@ -2,7 +2,7 @@
    esmtp.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2007 The Anubis Team.
+   Copyright (C) 2001-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -23,9 +23,9 @@
 
 #if defined(WITH_GSASL)
 
-ANUBIS_LIST *anubis_client_mech_list;     /* List of auth methods allowed by
+ANUBIS_LIST anubis_client_mech_list;     /* List of auth methods allowed by
 					     the client */
-ANUBIS_LIST *anubis_encryption_mech_list; /* List of auth methods that require
+ANUBIS_LIST anubis_encryption_mech_list; /* List of auth methods that require
 					     using encrypted channel */
 char *anon_token;                         /* Anonymous token */
 char *authorization_id;       
@@ -39,13 +39,13 @@ char *auth_passcode;
 char *auth_realm;
 
 void
-anubis_set_client_mech_list (ANUBIS_LIST *list)
+anubis_set_client_mech_list (ANUBIS_LIST list)
 {
   anubis_set_mech_list (&anubis_client_mech_list, list);
 }
 
 void
-anubis_set_encryption_mech_list (ANUBIS_LIST *list)
+anubis_set_encryption_mech_list (ANUBIS_LIST list)
 {
   anubis_set_mech_list (&anubis_encryption_mech_list, list);
 }
@@ -120,18 +120,22 @@ static char *
 get_reply (NET_STREAM str, int *code, char **buf, size_t *psize)
 {
   char *p;
-  
-  get_response_smtp (CLIENT, str, buf, psize);
+
+  if (recvline (CLIENT, str, buf, psize) == 0)
+    {
+      anubis_error (1, 0, _("unexpected eof in input"));
+    }
+      
   remcrlf (*buf);
   *code = strtoul (*buf, &p, 10);
   if (*p == 0 || *p == '\r')
     return p;
-
+  
   if (!isspace (*p))
     {
       anubis_error (1, 0, _("Malformed or unexpected reply"));
     }
-
+  
   while (*p && isspace (*p))
     p++;
   return p;
@@ -223,12 +227,12 @@ do_gsasl_auth (NET_STREAM *pstr, Gsasl *ctx, char *mech)
 }
 
 int
-esmtp_auth (NET_STREAM *pstr, char *input)
+esmtp_auth (NET_STREAM *pstr, const char *input)
 {
   Gsasl *ctx;
   int rc;
-  ANUBIS_LIST *isect;
-  ANUBIS_LIST *mech_list = auth_method_list (input);
+  ANUBIS_LIST isect;
+  ANUBIS_LIST mech_list = auth_method_list (input);
   char *mech;
   
   if (list_count (mech_list) == 0)
@@ -302,7 +306,7 @@ esmtp_auth (NET_STREAM *pstr, char *input)
 }
 #else
 int
-esmtp_auth (NET_STREAM *pstr, char *input)
+esmtp_auth (NET_STREAM *pstr, const char *input)
 {
   anubis_warning (0, _("ESMTP AUTH is not supported"));
   return 1;
