@@ -2,7 +2,7 @@
    extern.h
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 The Anubis Team.
+   Copyright (C) 2001-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -59,19 +59,10 @@ struct session_struct
 #endif				/* USE_SOCKS_PROXY */
 };
 
-struct message_struct
-{
-  ANUBIS_LIST *commands;	/* Associative list of SMTP commands */
-  ANUBIS_LIST *header;		/* Associative list of RFC822 headers */
-  ANUBIS_LIST *mime_hdr;	/* List of lines before the first boundary
-				   marker */
-  char *body;			/* Message body */
-  char *boundary;		/* Additional data */
-};
-
 #ifdef USE_SSL
 struct secure_struct
 {
+  char *prio;
   char *cafile;
   char *cert;
   char *key;
@@ -110,6 +101,10 @@ extern char **x_argv;
 
 extern char *incoming_mail_rule;
 extern char *outgoing_mail_rule;
+extern char *smtp_command_rule;
+
+extern char *log_tag;
+extern int log_facility;
 
 extern char *from_address;
 
