@@ -2,7 +2,7 @@
    gdbm.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2003, 2004, 2007, 2008 The Anubis Team.
+   Copyright (C) 2003-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -34,7 +34,7 @@
 
 static int
 gdbm_db_open (void **dp, ANUBIS_URL * url, enum anubis_db_mode mode,
-	      char **errp)
+	      char const **errp)
 {
   GDBM_FILE dbf;
   int flags;
@@ -118,7 +118,7 @@ gdbm_db_get (void *d, const char *keystr, ANUBIS_USER * rec, int *errp)
 }
 
 static int
-gdbm_db_list (void *d, ANUBIS_LIST * list, int *ecode)
+gdbm_db_list (void *d, ANUBIS_LIST  list, int *ecode)
 {
   datum key, content;
 
