@@ -2,7 +2,7 @@
    help.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2007 The Anubis Team.
+   Copyright (C) 2001-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -40,9 +40,9 @@ static char *config_opts[] = {
 #ifdef WITH_GUILE
   "GUILE",
 #endif				/* WITH_GUILE */
-#ifdef HAVE_TLS
+#ifdef USE_GNUTLS
   "GNUTLS",
-#endif				/* HAVE_TLS */
+#endif				/* USE_GNUTLS */
 #ifdef HAVE_GPG
   "GPG",
 #endif				/* HAVE_GPG */
