@@ -1,6 +1,6 @@
 /*
    GNU Anubis -- an SMTP message submission daemon.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 The Anubis Team.
+   Copyright (C) 2001-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,9 +25,9 @@ const char version[] = "GNU Anubis v" VERSION;
 
 struct options_struct options;
 struct session_struct session;
-#if defined(HAVE_TLS) || defined(HAVE_SSL)
+#if defined(USE_GNUTLS)
 struct secure_struct secure;
-#endif /* HAVE_TLS or HAVE_SSL */
+#endif /* USE_GNUTLS */
 
 unsigned long topt;
 NET_STREAM remote_client;
@@ -36,17 +36,7 @@ NET_STREAM remote_server;
 char *anubis_domain;      /* Local domain for EHLO in authentication mode */
 char *incoming_mail_rule; /* Name of section for incoming mail processing */
 char *outgoing_mail_rule; /* Name of section for outgoing mail processing */
-
-#ifdef WITH_GUILE
-void
-anubis_core (void)
-{
-  char *argv[] = { "anubis", NULL };
-  scm_boot_guile (1, argv, anubis_boot, NULL);
-}
-#else
-# define anubis_core() anubis(NULL)
-#endif /* WITH_GUILE */
+char *smtp_command_rule;  /* Name of section for rewriting SMTP commands */
 
 void
 xalloc_die ()
@@ -99,6 +89,7 @@ main (int argc, char *argv[])
   anubis_getlogin (&session.supervisor);
   assign_string (&incoming_mail_rule, "INCOMING");
   assign_string (&outgoing_mail_rule, "RULE");
+  assign_string (&smtp_command_rule, "SMTP");
   
   /*
      Initialize various database formats
@@ -161,17 +152,14 @@ main (int argc, char *argv[])
   init_ssl_libs ();
 #endif /* USE_SSL */
 
+#ifdef WITH_GUILE
+  init_guile ();
+#endif
+  
   /*
      Enter the main core...
    */
 
-  anubis_core ();
-  return 0;
-}
-
-void
-anubis (char *arg)
-{
   if (anubis_mode == anubis_mda)  /* Mail Delivery Agent */
     mda ();
   else if (topt & T_STDINOUT)     /* stdin/stdout */
@@ -180,12 +168,15 @@ anubis (char *arg)
     {				  /* daemon */
       int sd_bind;
       sd_bind = bind_and_listen (session.anubis, session.anubis_port);
+      
       if (topt & T_FOREGROUND_INIT)
 	topt |= T_FOREGROUND;
       else
 	daemonize ();
       loop (sd_bind);
     }
+  return 0;
 }
 
+
 /* EOF */
