@@ -2,7 +2,7 @@
    tunnel.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001, 2002, 2003, 2004, 2005, 2007, 2008 The Anubis Team.
+   Copyright (C) 2001-2014 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -25,16 +25,17 @@
 #define obstack_chunk_free free
 #include <obstack.h>
 
-static int transfer_command (MESSAGE *, char *);
-static int process_command (MESSAGE *, char *);
-static void process_data (MESSAGE *);
-static int handle_ehlo (char *, char **);
+static int transfer_command (MESSAGE);
+static int process_command (MESSAGE, char *);
+static void process_data (MESSAGE);
+static int handle_ehlo (ANUBIS_SMTP_REPLY );
 
 
 
 static char *smtp_ehlo_domain_name = NULL;
+static ANUBIS_SMTP_REPLY ehlo_reply = NULL;
 
-static char *
+char *
 get_ehlo_domain (void)
 {
   return smtp_ehlo_domain_name ? smtp_ehlo_domain_name : get_localname ();
@@ -50,10 +51,11 @@ get_ehlo_domain (void)
    and sent in multiline lines. */
 
 static void
-get_boundary (MESSAGE * msg, char *line)
+get_boundary (MESSAGE msg, char *line)
 {
   char boundary_buf[LINEBUFFER + 1], *p;
-
+  char *boundary;
+  
   if (strncmp (line, "Content-Type:", 13))
     return;
 
@@ -74,12 +76,13 @@ get_boundary (MESSAGE * msg, char *line)
       if (*q)
 	*q = 0;
     }
-  msg->boundary = xmalloc (strlen (p) + 3);
-  sprintf (msg->boundary, "--%s", p);
+  boundary = xmalloc (strlen (p) + 3);
+  sprintf (boundary, "--%s", p);
+  message_replace_boundary (msg, boundary);
 }
 
 static void
-add_header (ANUBIS_LIST * list, char *line)
+add_header (ANUBIS_LIST list, char *line)
 {
   ASSOC *asc = header_assoc (line);
   list_append (list, asc);
@@ -101,7 +104,7 @@ add_header (ANUBIS_LIST * list, char *line)
 }
 
 void
-collect_headers (MESSAGE * msg, char *line)
+collect_headers (MESSAGE msg, char *line)
 {
   char *buf = NULL;
   size_t size = 0;
@@ -123,9 +126,9 @@ collect_headers (MESSAGE * msg, char *line)
 	{
 	  if (line)
 	    {
-	      if (!(topt & T_ENTIRE_BODY) && msg->boundary == NULL)
+	      if (!(topt & T_ENTIRE_BODY) && message_get_boundary (msg))
 		get_boundary (msg, line);
-	      add_header (msg->header, line);
+	      add_header (message_get_header (msg), line);
 	      xfree (line);
 	    }
 	  if (buf[0] == 0)
@@ -150,7 +153,7 @@ write_header_line (NET_STREAM sd_server, char *line)
 }
 
 static void
-write_assoc (NET_STREAM sd_server, ASSOC * entry)
+write_assoc (NET_STREAM sd_server, ASSOC *entry)
 {
   if (entry->key)
     {
@@ -163,10 +166,10 @@ write_assoc (NET_STREAM sd_server, ASSOC * entry)
 }
 
 void
-send_header (NET_STREAM sd_server, ANUBIS_LIST * list)
+send_header (NET_STREAM sd_server, ANUBIS_LIST list)
 {
   ASSOC *p;
-  ITERATOR *itr = iterator_create (list);
+  ITERATOR itr = iterator_create (list);
 
   for (p = iterator_first (itr); p; p = iterator_next (itr))
     write_assoc (sd_server, p);
@@ -174,10 +177,10 @@ send_header (NET_STREAM sd_server, ANUBIS_LIST * list)
 }
 
 void
-send_string_list (NET_STREAM sd_server, ANUBIS_LIST * list)
+send_string_list (NET_STREAM sd_server, ANUBIS_LIST list)
 {
   char *p;
-  ITERATOR *itr = iterator_create (list);
+  ITERATOR itr = iterator_create (list);
 
   for (p = iterator_first (itr); p; p = iterator_next (itr))
     write_header_line (sd_server, p);
@@ -202,7 +205,7 @@ send_string_list (NET_STREAM sd_server, ANUBIS_LIST * list)
 #define ST_DONE  3
 
 void
-collect_body (MESSAGE * msg)
+collect_body (MESSAGE msg)
 {
   int nread;
   char *buf = NULL;
@@ -210,12 +213,10 @@ collect_body (MESSAGE * msg)
   struct obstack stk;
   int state = 0;
   int len;
-
-  if (msg->boundary)
-    {
-      len = strlen (msg->boundary);
-      msg->mime_hdr = list_create ();
-    }
+  const char *boundary = message_get_boundary (msg);
+  
+  if (boundary)
+    len = strlen (boundary);
   obstack_init (&stk);
   while (state != ST_DONE
 	 && (nread = recvline (SERVER, remote_client, &buf, &size)))
@@ -225,12 +226,12 @@ collect_body (MESSAGE * msg)
       if (strcmp (buf, ".") == 0)	/* EOM */
 	break;
 
-      if (msg->boundary)
+      if (boundary)
 	{
 	  switch (state)
 	    {
 	    case ST_INIT:
-	      if (strcmp (buf, msg->boundary) == 0)
+	      if (strcmp (buf, boundary) == 0)
 		state = ST_HDR;
 	      break;
 
@@ -238,11 +239,11 @@ collect_body (MESSAGE * msg)
 	      if (buf[0] == 0)
 		state = ST_BODY;
 	      else
-		list_append (msg->mime_hdr, strdup (buf));
+		message_append_mime_header (msg, buf);
 	      break;
 
 	    case ST_BODY:
-	      if (strncmp (buf, msg->boundary, len) == 0)
+	      if (strncmp (buf, boundary, len) == 0)
 		state = ST_DONE;
 	      else
 		{
@@ -259,39 +260,41 @@ collect_body (MESSAGE * msg)
     }
   free (buf);
   obstack_1grow (&stk, 0);
-  msg->body = strdup (obstack_finish (&stk));
+  /* FIXME: Use message_proc_body to avoid spurious reallocations */
+  message_replace_body (msg, xstrdup (obstack_finish (&stk)));
   obstack_free (&stk, NULL);
 }
 
 void
-send_body (MESSAGE * msg, NET_STREAM sd_server)
+send_body (MESSAGE msg, NET_STREAM sd_server)
 {
-  char *p;
-
-  if (msg->boundary)
+  const char *p;
+  const char *boundary = message_get_boundary (msg);
+  
+  if (boundary)
     {
-      swrite (CLIENT, sd_server, msg->boundary);
+      swrite (CLIENT, sd_server, boundary);
       send_eol (CLIENT, sd_server);
-      send_string_list (sd_server, msg->mime_hdr);
+      send_string_list (sd_server, message_get_mime_header (msg));
       send_eol (CLIENT, sd_server);
     }
 
-  for (p = msg->body; *p;)
+  for (p = message_get_body (msg); *p;)
     {
-      char *q = strchr (p, '\n');
-      if (q)
-	*q++ = 0;
+      size_t len = strcspn (p, "\n");
+      
+      swrite_n (CLIENT, sd_server, p, len);
+      send_eol (CLIENT, sd_server);
+      p += len;
+      if (*p)
+	p++;
       else
-	q = p + strlen (p);
-
-      swrite (CLIENT, sd_server, p);
-      send_eol (CLIENT, sd_server);
-      p = q;
+	break;
     }
-
-  if (msg->boundary)
+      
+  if (boundary)
     {
-      swrite (CLIENT, sd_server, msg->boundary);
+      swrite (CLIENT, sd_server, boundary);
       send_eol (CLIENT, sd_server);
     }
 }
@@ -306,81 +309,79 @@ smtp_session_transparent (void)
 {
   char *command = NULL;
   size_t size = 0;
+  ANUBIS_SMTP_REPLY reply;
   MESSAGE msg;
 
   /*
      First of all, transfer a welcome message.
    */
 
+  reply = smtp_reply_new ();
   info (VERBOSE, _("Transferring messages..."));
-  get_response_smtp (CLIENT, remote_server, &command, &size);
+  smtp_reply_get (CLIENT, remote_server, reply);
 
-  if (command &&
-      strncmp (command, "220 ", 4) == 0 && strstr (command, version) == 0)
+  if (smtp_reply_code_eq (reply, "220")
+      && !smtp_reply_has_string (reply, 0, version, NULL))
     {
-      char *ptr = 0;
-      char *banner_ptr = 0;
-      char host[65];
-      char banner_backup[LINEBUFFER + 1];
-
-      safe_strcpy (banner_backup, command);
-
-      if ((banner_ptr = strchr (banner_backup, ' ')))
-	{
-	  banner_ptr++;
-	  safe_strcpy (host, banner_ptr);
-	  if ((ptr = strchr (host, ' ')))
-	    *ptr = '\0';
-	  do
-	    {
-	      banner_ptr++;
-	    }
-	  while (*banner_ptr != ' ');
-	  banner_ptr++;
-	  command = xmalloc (4 + strlen (host) + 2 + strlen (version) + 2
-			     + strlen (banner_ptr) + 1);
-	  sprintf (command, "220 %s (%s) %s", host, version, banner_ptr);
-	}
+      char *banner_copy;
+      char *host;
+      char *rest;
+      char *str;
+      
+      smtp_reply_get_line (reply, 0, &banner_copy, 0);
+      for (host = banner_copy + 4; *host && *host == ' '; host++);
+      rest = strchr (host, ' ');
+      if (rest)
+	*rest++ = 0;
+      else
+	rest = "";
+      
+      str = xmalloc (4 + strlen (host) + 2 + strlen (version) + 2
+		     + strlen (rest) + 1);
+      sprintf (str, "%s (%s) %s", host, version, rest);
+      
+      smtp_reply_replace_line (reply, 0, str);
+      free (str);
+      free (banner_copy);
     }
-  swrite (SERVER, remote_client, command);
-
+  swrite (SERVER, remote_client, smtp_reply_string (reply));
+  smtp_reply_free (reply);
+  
   /*
      Then process the commands...
    */
 
-  message_init (&msg);
+  msg = message_new ();
   while (recvline (SERVER, remote_client, &command, &size))
     {
       remcrlf (command);
 
-      if (process_command (&msg, command))
+      if (process_command (msg, command))
 	continue;
 
-      if (transfer_command (&msg, command) == 0)
+      if (transfer_command (msg) == 0)
 	break;
     }
   free (command);
 
-  message_free (&msg);
+  message_free (msg);
 }
 
 void
 smtp_begin (void)
 {
-  char *command = NULL;
-  size_t size;
-  char *reply;
+  ANUBIS_SMTP_REPLY reply;
   
   /* first get an mta banner */
-  get_response_smtp (CLIENT, remote_server, &command, &size);
+  reply = smtp_reply_new ();
+  smtp_reply_get (CLIENT, remote_server, reply);
 
   /* now send the ehlo command */
   swrite (CLIENT, remote_server, "EHLO ");
   swrite (CLIENT, remote_server, get_ehlo_domain ());
   send_eol (CLIENT, remote_server);
-  handle_ehlo (command, &reply);
-  free (command);
-  free (reply);
+  handle_ehlo (reply);
+  smtp_reply_free (reply);
 }
 
 void
@@ -394,19 +395,19 @@ smtp_session (void)
   smtp_begin ();
   info (VERBOSE, _("Transferring messages..."));
 
-  message_init (&msg);
+  msg = message_new ();
   while (recvline (SERVER, remote_client, &command, &size))
     {
       remcrlf (command);
       
-      if (process_command (&msg, command))
+      if (process_command (msg, command))
 	continue;
 
-      if (transfer_command (&msg, command) == 0)
+      if (transfer_command (msg) == 0)
 	break;
     }
   free (command);
-  message_free (&msg);
+  message_free (msg);
 }
 
 /********************
@@ -414,7 +415,7 @@ smtp_session (void)
 *********************/
 
 static void
-save_command (MESSAGE * msg, char *line)
+save_command (MESSAGE msg, const char *line)
 {
   int i;
   ASSOC *asc = xmalloc (sizeof (*asc));
@@ -425,14 +426,14 @@ save_command (MESSAGE * msg, char *line)
   if (i == 4)
     {
       char *expect = NULL;
-      if (memcmp (line, "mail", 4) == 0)
+      if (strncasecmp (line, "mail", 4) == 0)
 	expect = " from:";
-      else if (memcmp (line, "rcpt", 4) == 0)
+      else if (strncasecmp (line, "rcpt", 4) == 0)
 	expect = " to:";
       if (expect)
 	{
 	  int n = strlen (expect);
-	  if (strncmp (&line[i], expect, n) == 0)
+	  if (strncasecmp (&line[i], expect, n) == 0)
 	    i += n;
 	}
     }
@@ -440,6 +441,8 @@ save_command (MESSAGE * msg, char *line)
   asc->key = xmalloc (i + 1);
   memcpy (asc->key, line, i);
   asc->key[i] = 0;
+  make_uppercase (asc->key);
+  
   for (; line[i] && isspace ((u_char) line[i]); i++)
     ;
 
@@ -447,7 +450,7 @@ save_command (MESSAGE * msg, char *line)
     asc->value = strdup (&line[i]);
   else
     asc->value = NULL;
-  list_append (msg->commands, asc);
+  message_add_command (msg, asc);
 }
 
 static int
@@ -480,26 +483,26 @@ handle_starttls (char *command)
   if (!(topt & T_LOCAL_MTA))
     {
       NET_STREAM stream;
-      char *reply = NULL;
-      size_t size = 0;
+      ANUBIS_SMTP_REPLY reply = smtp_reply_new ();
+      const char *rstr;
+      
       swrite (CLIENT, remote_server, "STARTTLS" CRLF);
+      smtp_reply_get (CLIENT, remote_server, reply);
 
-      get_response_smtp (CLIENT, remote_server, &reply, &size);
-
-      if (!isdigit ((unsigned char) reply[0])
-	  || (unsigned char) reply[0] > '3')
+      rstr = smtp_reply_string (reply);
+      /* FIXME: Use smtp_reply_eq */
+      if (!isdigit ((unsigned char) rstr[0])
+	  || (unsigned char) rstr[0] > '3')
 	{
-	  remcrlf (reply);
-	  info (VERBOSE, _("WARNING: %s"), reply);
-	  free (reply);
+	  /* FIXME: Display complete response */
+	  info (VERBOSE, _("WARNING: %s"), smtp_reply_line (reply, 0));
+	  smtp_reply_free (reply);
 	  anubis_error (0, 0, _("STARTTLS command failed."));
 	  return 0;
 	}
-      free (reply);
+      smtp_reply_free (reply);
 
-      stream = start_ssl_client (remote_server,
-				 secure.cafile,
-				 options.termlevel > NORMAL);
+      stream = start_ssl_client (remote_server, options.termlevel > NORMAL);
       if (!stream)
 	return 0;
       remote_server = stream;
@@ -537,10 +540,7 @@ handle_starttls (char *command)
     check_filemode (secure.key);
 
   swrite (SERVER, remote_client, "220 2.0.0 Ready to start TLS" CRLF);
-  stream = start_ssl_server (remote_client,
-			     secure.cafile,
-			     secure.cert,
-			     secure.key, options.termlevel > NORMAL);
+  stream = start_ssl_server (remote_client, options.termlevel > NORMAL);
   if (!stream)
     {
       swrite (SERVER, remote_client, "454 4.3.3 TLS not available" CRLF);
@@ -556,33 +556,29 @@ handle_starttls (char *command)
 }
 
 static int
-process_command (MESSAGE * msg, char *command)
+process_command (MESSAGE msg, char *command)
 {
-  char buf[LINEBUFFER + 1];
-
-  safe_strcpy (buf, command);	/* make a back-up */
-  make_lowercase (buf);
-  save_command (msg, buf);
-
-  if (!strncmp (buf, "starttls", 8))
-    return handle_starttls (buf);
-  else if (!strncmp (buf, "xdatabase", 9))
-    return xdatabase (buf + 9);
-
+  save_command (msg, command);
+  if (!strncasecmp (command, "starttls", 8))
+    return handle_starttls (command);
+  else if (!strncasecmp (command, "xdatabase", 9))
+    return xdatabase (command + 9);
   return 0;
 }
 
 void
-set_ehlo_domain (char *domain)
+set_ehlo_domain (const char *domain, size_t len)
 {
   xfree (smtp_ehlo_domain_name);
-  smtp_ehlo_domain_name = strdup (domain);
+  smtp_ehlo_domain_name = xmalloc (len + 1);
+  memcpy (smtp_ehlo_domain_name, domain, len);
+  smtp_ehlo_domain_name[len] = 0;
 }
 
 static void
-save_ehlo_domain (char *command)
+save_ehlo_domain (const char *command)
 {
-  char *p, *endp;
+  const char *p, *endp;
   
   for (p = command + 5 /* length of EHLO + initial space */;
        *p && isspace (*p);
@@ -591,35 +587,33 @@ save_ehlo_domain (char *command)
 
   for (endp = p + strlen (p) - 1; endp >= p && isspace(*endp); endp--)
     ;
-  endp[1] = 0;
-  set_ehlo_domain (p);
+  set_ehlo_domain (p, endp - p + 1);
 }
 
 static int
-handle_ehlo (char *command, char **reply)
+handle_ehlo (ANUBIS_SMTP_REPLY reply)
 {
-  size_t reply_size = 0;
-  
-  *reply = NULL;
   if (!smtp_ehlo_domain_name)
-    save_ehlo_domain (command);
-    
-  get_response_smtp (CLIENT, remote_server, reply, &reply_size);
+    save_ehlo_domain (smtp_reply_line (reply, 0));
+		   
+  smtp_reply_free (ehlo_reply);
+  ehlo_reply = smtp_reply_new ();
+  smtp_reply_get (CLIENT, remote_server, ehlo_reply);
 
-  if (strstr (*reply, "STARTTLS"))
+  if (smtp_reply_has_capa (ehlo_reply, "STARTTLS", NULL))
     topt |= T_STARTTLS;		/* Yes, we can use the TLS/SSL
 				   encryption. */
 
-  xdatabase_capability (reply, &reply_size);
+  xdatabase_capability (ehlo_reply);
 
 #ifdef USE_SSL
   if ((topt & T_SSL_ONEWAY)
       && (topt & T_STARTTLS) && !(topt & T_SSL_FINISHED))
     {
       NET_STREAM stream;
-      char *newreply = NULL;
-      size_t nsize = 0;
-
+      ANUBIS_SMTP_REPLY newreply = smtp_reply_new ();
+      const char *rstr;
+      
       /*
          The 'ONEWAY' method is used when your MUA doesn't
          support the TLS/SSL, but your MTA does.
@@ -631,27 +625,26 @@ handle_ehlo (char *command, char **reply)
       info (NORMAL,
 	    _("Using TLS/SSL encryption between Anubis and remote MTA only..."));
       swrite (CLIENT, remote_server, "STARTTLS" CRLF);
-      get_response_smtp (CLIENT, remote_server, &newreply, &nsize);
+      smtp_reply_get (CLIENT, remote_server, newreply);
 
-      if (!isdigit ((unsigned char) newreply[0])
-	  || (unsigned char) newreply[0] > '3')
+      rstr = smtp_reply_string (newreply);
+      if (!isdigit ((unsigned char) rstr[0])
+	  || (unsigned char) rstr[0] > '3')
 	{
-	  remcrlf (newreply);
-	  info (VERBOSE, _("WARNING: %s"), newreply);
-	  free (newreply);
+	  info (VERBOSE, _("WARNING: %s"), smtp_reply_line (newreply, 0));
+	  smtp_reply_free (newreply);
 	  anubis_error (0, 0, _("STARTTLS (ONEWAY) command failed."));
 	  topt &= ~T_SSL_ONEWAY;
-	  swrite (SERVER, remote_client, *reply);
+	  swrite (SERVER, remote_client, smtp_reply_string (ehlo_reply));
 	  return 1;
 	}
+      smtp_reply_free (newreply);
 
-      stream = start_ssl_client (remote_server,
-				 secure.cafile,
-				 options.termlevel > NORMAL);
+      stream = start_ssl_client (remote_server, options.termlevel > NORMAL);
       if (!stream)
 	{
 	  topt &= ~T_SSL_ONEWAY;
-	  swrite (SERVER, remote_client, *reply);
+	  swrite (SERVER, remote_client, smtp_reply_string (ehlo_reply));
 	  return 1;
 	}
 
@@ -665,7 +658,7 @@ handle_ehlo (char *command, char **reply)
       swrite (CLIENT, remote_server, "EHLO ");
       swrite (CLIENT, remote_server, get_ehlo_domain ());
       send_eol (CLIENT, remote_server);
-      get_response_smtp (CLIENT, remote_server, reply, &reply_size);
+      smtp_reply_get (CLIENT, remote_server, ehlo_reply);
     }
 #endif /* USE_SSL */
 
@@ -677,12 +670,9 @@ handle_ehlo (char *command, char **reply)
 
   if ((topt & T_STARTTLS) && (!(topt & T_SSL) || (topt & T_SSL_ONEWAY)))
     {
-      char *starttls1 = "250-STARTTLS";
-      char *starttls2 = "STARTTLS";
-      if (strstr (*reply, starttls1))
-	remline (*reply, starttls1);
-      else if (strstr (*reply, starttls2))
-	remline (*reply, starttls2);
+      size_t n;
+      if (smtp_reply_has_capa (ehlo_reply, "STARTTLS", &n))
+	smtp_reply_remove_line (ehlo_reply, n);
     }
 
   /*
@@ -691,94 +681,95 @@ handle_ehlo (char *command, char **reply)
 
   if (topt & T_ESMTP_AUTH)
     {
-      char *p = strstr (*reply, "AUTH ");
-      if (p && (p[-1] == '-' || p[-1] == ' ') && memcmp (p-4, "250", 3) == 0)
+      size_t n;
+      if (smtp_reply_has_capa (ehlo_reply, "AUTH", &n))
 	{
-	  char *q = strchr (p, '\r');
-	  if (q)
-	    *q++ = 0;
-	  esmtp_auth (&remote_server, p + 5);
-	  if (q)
-	    memmove (p - 4, q + 1, strlen (q + 1) + 1);
+	  if (!(topt & T_ESMTP_AUTH_DELAYED))
+	    {
+	      const char *p = smtp_reply_line (ehlo_reply, n);
+	      esmtp_auth (&remote_server, p + 9);
+	      smtp_reply_remove_line (ehlo_reply, n);
+	      topt &= ~T_ESMTP_AUTH;
+	    }
 	}
+      else
+	topt &= ~T_ESMTP_AUTH;
     }
-  
+
+  smtp_reply_set (reply, smtp_reply_string (ehlo_reply));
+
   return 0;
 }
 
-#define CMD_MAIL_FROM "mail from:"
-#define CMD_RCPT_TO "rcpt to:"
-
-static void
-strip_inter_ws(char *buf, size_t len, size_t pfxlen)
-{
-  if (isspace (buf[pfxlen]))
-    {
-      size_t i;
-      for (i = pfxlen; i < len && isspace (buf[i]); i++)
-	;
-      memmove (buf + pfxlen, buf + i, len - i + 1);
-    }
-}
-
 static int
-is_prefix(char *buf, char *pfx)
+transfer_command (MESSAGE msg)
 {
-  while (*pfx)
-    {
-      int b = *buf++;
-      int c = *pfx++;
-      
-      if (!b || toupper (c) != toupper (b))
-	return 0;
-    }
-  return 1;
-}
-
-static int
-transfer_command (MESSAGE * msg, char *command)
-{
-  char *reply = NULL;
-  size_t reply_size = 0;
   char *buf = NULL;
   int rc = 1; /* OK */
-  size_t len;
+  ANUBIS_SMTP_REPLY reply = smtp_reply_new ();
+  const char *rstr;
+  ASSOC *asc;
+  char *command;
+  int len;
   
+  rcfile_call_section (CF_CLIENT, smtp_command_rule, "SMTP", NULL, msg);
+  asc = list_tail_item (message_get_commands (msg));
+  if (!asc->value)
+    command = strdup (asc->key);
+  else if (strcasecmp (asc->key, "mail from:") == 0)
+    {
+      if (topt & T_ESMTP_AUTH)
+	{
+	  size_t n;
+	  if (smtp_reply_has_capa (ehlo_reply, "AUTH", &n))
+	    {
+	      const char *p = smtp_reply_line (ehlo_reply, n);
+	      esmtp_auth (&remote_server, p + 9);
+	      smtp_reply_remove_line (ehlo_reply, n);
+	    }
+	  topt &= ~T_ESMTP_AUTH;
+	}
+      asprintf (&command, "%s%s", asc->key, asc->value);
+    }
+  else if (strcasecmp (asc->key, "rcpt to:") == 0)
+    asprintf (&command, "%s%s", asc->key, asc->value);
+  else
+    asprintf (&command, "%s %s", asc->key, asc->value);
+	    
   assign_string (&buf, command);
   make_lowercase (buf);
 
-  len = strlen (command);
-  if (len > sizeof (CMD_MAIL_FROM)
-      && is_prefix (command, CMD_MAIL_FROM))
-    strip_inter_ws (command, len, sizeof (CMD_MAIL_FROM) - 1);
-  else if (len > sizeof (CMD_RCPT_TO)
-	   && is_prefix (command, CMD_RCPT_TO))
-    strip_inter_ws (command, len, sizeof (CMD_RCPT_TO) - 1);
-  
   swrite (CLIENT, remote_server, command);
   swrite (CLIENT, remote_server, CRLF);
 
   if (!strncmp (buf, "ehlo", 4))
     {
-      if (handle_ehlo (command, &reply))
+      smtp_reply_set (reply, command);
+      if (handle_ehlo (reply))
 	{
-	  free (reply);
+	  smtp_reply_free (reply);
+	  free (command);
 	  return 0;
 	}
     }
   else
-    get_response_smtp (CLIENT, remote_server, &reply, &reply_size);
+    smtp_reply_get (CLIENT, remote_server, reply);
+  
+  swrite (SERVER, remote_client, smtp_reply_string (reply));
 
-  swrite (SERVER, remote_client, reply);
-
-  if (isdigit ((unsigned char) reply[0]) && (unsigned char) reply[0] < '4')
+  rstr = smtp_reply_line_ptr (reply, 0);
+  len = strcspn (rstr, "\r\n");
+  info (NORMAL, "%s: %s <=> %*.*s",
+	message_id (msg), command,
+	len, len, rstr);
+  rstr = smtp_reply_string (reply);
+  if (isdigit ((unsigned char) rstr[0]) && (unsigned char) rstr[0] < '4')
     {
       if (strncmp (buf, "quit", 4) == 0)
 	rc = 0;		/* The QUIT command */
       else if (strncmp (buf, "rset", 4) == 0)
 	{
-	  message_free (msg);
-	  message_init (msg);
+	  message_reset (msg);
 	}
       else if (strncmp (buf, "data", 4) == 0)
 	{
@@ -786,12 +777,13 @@ transfer_command (MESSAGE * msg, char *command)
 	}
     }
   free (buf);
-  free (reply);
+  free (command);
+  smtp_reply_free (reply);
   return rc;
 }
 
 void
-process_data (MESSAGE * msg)
+process_data (MESSAGE msg)
 {
   char *buf = NULL;
   size_t size = 0;
@@ -801,9 +793,9 @@ process_data (MESSAGE * msg)
   collect_headers (msg, NULL);
   collect_body (msg);
 
-  rcfile_call_section (CF_CLIENT, outgoing_mail_rule, NULL, msg);
+  rcfile_call_section (CF_CLIENT, outgoing_mail_rule, "RULE", NULL, msg);
 
-  transfer_header (msg->header);
+  transfer_header (message_get_header (msg));
   transfer_body (msg);
 
   if (recvline (CLIENT, remote_server, &buf, &size))
@@ -812,16 +804,17 @@ process_data (MESSAGE * msg)
       
       swrite (SERVER, remote_client, buf);
       send_eol (SERVER, remote_client);
+
+      info (NORMAL, "%s: dot <=> %s", message_id (msg), buf);
     }
   free (buf);
 
-  message_free (msg);
-  message_init (msg);
+  message_reset (msg);
   alarm (0);
 }
 
 void
-transfer_header (ANUBIS_LIST * header_buf)
+transfer_header (ANUBIS_LIST header_buf)
 {
   send_header (remote_server, header_buf);
   send_eol (CLIENT, remote_server);
@@ -848,9 +841,9 @@ raw_transfer (void)
 }
 
 void
-transfer_body (MESSAGE * msg)
+transfer_body (MESSAGE msg)
 {
-  if (msg->boundary)
+  if (message_get_boundary (msg))
     {
       send_body (msg, remote_server);
 
